==> apache-paimon-0.9.0-src.tgz: paimon-0.9.0/paimon-spark/paimon-spark-common/src/main/java/org/apache/paimon/spark/procedure/CreateTagProcedure.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.paimon.spark.procedure;

import org.apache.paimon.utils.TimeUtils;

import org.apache.spark.sql.catalyst.InternalRow;
import org.apache.spark.sql.connector.catalog.Identifier;
import org.apache.spark.sql.connector.catalog.TableCatalog;
import org.apache.spark.sql.types.DataTypes;
import org.apache.spark.sql.types.Metadata;
import org.apache.spark.sql.types.StructField;
import org.apache.spark.sql.types.StructType;

import java.time.Duration;

import static org.apache.spark.sql.types.DataTypes.LongType;
import static org.apache.spark.sql.types.DataTypes.StringType;

/** A procedure to create a tag. */
public class CreateTagProcedure extends BaseProcedure {

    private static final ProcedureParameter[] PARAMETERS =
            new ProcedureParameter[] {
                ProcedureParameter.required("table", StringType),
                ProcedureParameter.required("tag", StringType),
                ProcedureParameter.optional("snapshot", LongType),
                ProcedureParameter.optional("time_retained", StringType)
            };

    private static final StructType OUTPUT_TYPE =
            new StructType(
                    new StructField[] {
                        new StructField("result", DataTypes.BooleanType, true, Metadata.empty())
                    });

    protected CreateTagProcedure(TableCatalog tableCatalog) {
        super(tableCatalog);
    }

    @Override
    public ProcedureParameter[] parameters() {
        return PARAMETERS;
    }

    @Override
    public StructType outputType() {
        return OUTPUT_TYPE;
    }

    @Override
    public InternalRow[] call(InternalRow args) {
        Identifier tableIdent = toIdentifier(args.getString(0), PARAMETERS[0].name());
        String tag = args.getString(1);
        Long snapshot = args.isNullAt(2) ? null : args.getLong(2);
        Duration timeRetained =
                args.isNullAt(3) ? null : TimeUtils.parseDuration(args.getString(3));

        return modifyPaimonTable(
                tableIdent,
                table -> {
                    if (snapshot == null) {
                        table.createTag(tag, timeRetained);
                    } else {
                        table.createTag(tag, snapshot, timeRetained);
                    }
                    InternalRow outputRow = newInternalRow(true);
                    return new InternalRow[] {outputRow};
                });
    }

    public static ProcedureBuilder builder() {
        return new BaseProcedure.Builder<CreateTagProcedure>() {
            @Override
            public CreateTagProcedure doBuild() {
                return new CreateTagProcedure(tableCatalog());
            }
        };
    }

    @Override
    public String description() {
        return "CreateTagProcedure";
    }
}

==> apache-paimon-1.0.0-src.tgz: paimon-1.0.0/paimon-spark/paimon-spark-common/src/main/java/org/apache/paimon/spark/procedure/CreateTagProcedure.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.paimon.spark.procedure;

import org.apache.paimon.table.Table;

import org.apache.spark.sql.connector.catalog.TableCatalog;

import java.time.Duration;

/** A procedure to create a tag. */
public class CreateTagProcedure extends CreateOrReplaceTagBaseProcedure {

    private CreateTagProcedure(TableCatalog tableCatalog) {
        super(tableCatalog);
    }

    @Override
    void createOrReplaceTag(Table table, String tagName, Long snapshotId, Duration timeRetained) {
        if (snapshotId == null) {
            table.createTag(tagName, timeRetained);
        } else {
            table.createTag(tagName, snapshotId, timeRetained);
        }
    }

    public static ProcedureBuilder builder() {
        return new BaseProcedure.Builder<CreateTagProcedure>() {
            @Override
            public CreateTagProcedure doBuild() {
                return new CreateTagProcedure(tableCatalog());
            }
        };
    }

    @Override
    public String description() {
        return "CreateTagProcedure";
    }
}
